package com.gree.mall.contest.logic.admin;

import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.gree.mall.contest.bean.admin.AdminFastEntryVO;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.zfire.QueryParamBean;
import com.gree.mall.contest.bean.zfire.ZfireParamBean;
import com.gree.mall.contest.commonmapper.AdminMapper;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.AdminFastEntry;
import com.gree.mall.contest.plus.service.AdminFastEntryService;
import com.gree.mall.contest.utils.zfire.FieldUtils;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;

@Service
@Slf4j
@RequiredArgsConstructor
public class AdminFastEntryLogic {

    private final AdminMapper adminMapper;
    private final AdminFastEntryService adminFastEntryService;
    private final CommonLogic commonLogic;

    /**
     * 列表
     */
    public IPage<AdminFastEntryVO> listPage(ZfireParamBean zfireParamBean) {
        final AdminUserCom adminUser = commonLogic.getAdminUser();
        if (CollectionUtil.isEmpty(zfireParamBean.getParams())) {
            zfireParamBean.setParams(new ArrayList<>());
        }
        QueryParamBean paramBean = new QueryParamBean();
        paramBean.setParam("a.admin_user_id")
                .setCompare("=")
                .setValue(adminUser.getAdminUserId());
        zfireParamBean.getParams().add(paramBean);
        zfireParamBean = FieldUtils.supplyParam(zfireParamBean);
        return adminMapper.queryAdminFastEntry(new Page(zfireParamBean.getPageNum(),zfireParamBean.getPageSize()),zfireParamBean);
    }

    /**
     * 新增
     */
    public void add(AdminFastEntry adminFastEntry) {
        final AdminUserCom adminUser = commonLogic.getAdminUser();
        adminFastEntry.setAdminUserId(adminUser.getAdminUserId());
        adminFastEntryService.save(adminFastEntry);
    }

    /**
     * 修改角色
     */
    @Transactional
    public void update(AdminFastEntry adminFastEntry) {
        adminFastEntryService.updateById(adminFastEntry);
    }

    /**
     * 删除角色
     */
    public void delete(String id) {
        adminFastEntryService.removeById(id);
    }

    /**
     * 详情
     */
    public AdminFastEntry detail(String id) {
        return adminFastEntryService.getById(id);
    }
}
