package com.gree.mall.contest.logic.admin;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.gree.mall.contest.bean.admin.AdminUserCom;
import com.gree.mall.contest.bean.admin.AdminWebsitTree;
import com.gree.mall.contest.constant.Constant;
import com.gree.mall.contest.exception.RemoteServiceException;
import com.gree.mall.contest.logic.common.CommonLogic;
import com.gree.mall.contest.plus.entity.*;
import com.gree.mall.contest.plus.service.*;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Slf4j
@Service
@RequiredArgsConstructor
public class AdminWebsitLogic {

    private final AdminWebsitService adminWebsitService;
    private final AdminWebsitRegionService adminWebsitRegionService;
    private final ServiceProviderService serviceProviderService;
    private final AdminUserService adminUserService;
    private final CommonLogic commonLogic;

    private final AdminRoleService adminRoleService;

    /**
     * 部门树
     */
    public List<AdminWebsitTree> tree(HttpServletRequest request, Boolean isRole){
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        List<AdminRole> adminRoles = adminRoleService.lambdaQuery().in(CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds())
                ,AdminRole::getServiceProviderId,adminUser.getAdminWebsitIds()
        )
                .list();
        List<String> websids = adminRoles.stream().map(AdminRole::getAdminWebsitId).collect(Collectors.toList());


        if (isRole){
            if (CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds()))
                websids.addAll(adminUser.getAdminWebsitIds());
            List<AdminWebsit> list = adminWebsitService.lambdaQuery().in(AdminWebsit::getWebsitId,websids)
                    .orderByAsc(AdminWebsit::getCreateTime)
                    .list();

            List<AdminWebsitTree> adminWebsitTrees = BeanUtil.copyToList(list, AdminWebsitTree.class);

            return adminWebsitTrees;
        }

        List<AdminWebsit> list = adminWebsitService.list();
        List<AdminWebsitTree> trees = new ArrayList<>();


        for(AdminWebsit adminWebsit : list){
            AdminWebsitTree adminWebsitTree = new AdminWebsitTree();
            BeanUtils.copyProperties(adminWebsit,adminWebsitTree);
            trees.add(adminWebsitTree);
        }
        List<AdminWebsitTree> parentList = trees.stream()
                .filter(x ->  x.getWebsitId().equals(adminUser.getAdminWebsitId())).collect(Collectors.toList());

        for(AdminWebsitTree adminWebsit:parentList){
            adminWebsit.setChildren(this.treeModule(trees,adminWebsit.getWebsitId()));
        }
        return parentList;
    }


    public List<AdminWebsit> list(Boolean isAll,Boolean status){
        AdminUserCom adminUser = commonLogic.getAdminUser();

        List<AdminWebsit> list = adminWebsitService.lambdaQuery()
                .eq(status != null,AdminWebsit::getStatus, status)
                .in(CollectionUtil.isNotEmpty(adminUser.getAdminWebsitIds()) && (isAll == null || !isAll),AdminWebsit::getWebsitId,adminUser.getAdminWebsitIds())
                .list();
        return list;
    }


    /**
     *  新增部门
     */
    @Transactional
    public void add(AdminWebsit adminWebsit, HttpServletRequest request) throws WxErrorException, RemoteServiceException {
        AdminUserCom adminUser = commonLogic.getAdminUser(request);

        Long count = adminWebsitService.lambdaQuery().eq(AdminWebsit::getName, adminWebsit.getName())
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称已存在");
        }
        AdminWebsit parent = adminWebsitService.getById(adminWebsit.getParentId());
        if(parent == null){
            throw new RemoteServiceException("请选择父部门");
        }
        List<AdminWebsit> list = new ArrayList<>();
        list.addAll(commonLogic.queryAllChild(list,adminWebsit.getWebsitId()));
        List<String> childWebsitIds = list.stream().map(AdminWebsit::getWebsitId).collect(Collectors.toList());
        if(childWebsitIds.contains(adminWebsit.getParentId())){
            throw new RemoteServiceException("父级部门不可以是自己或者自己的下级部门");
        }
        adminWebsit.setType(parent.getType());
        adminWebsit.setWebsitId(IdWorker.getIdStr());
        adminWebsitService.save(adminWebsit);
    }
    /**
     * 编辑部门
     */
    @Transactional
    public void update(AdminWebsit adminWebsit,HttpServletRequest request) throws RemoteServiceException, WxErrorException {
        AdminWebsit old = adminWebsitService.getById(adminWebsit.getWebsitId());
        if(Constant.SYS_WEBSIT_NAME.contains(old.getName())){
            throw new RemoteServiceException("不可操作当前部门信息");
        }
        Long count = adminWebsitService.lambdaQuery()
                .ne(AdminWebsit::getWebsitId,adminWebsit.getWebsitId())
                .eq(AdminWebsit::getName, adminWebsit.getName())
                .count();
        if(count > 0){
            throw new RemoteServiceException("部门名称或部门编号已存在");
        }
        if(adminWebsit.getParentId().equals("0")){
            throw new RemoteServiceException("请选择父部门");
        }
        AdminWebsit parent = adminWebsitService.getById(adminWebsit.getParentId());
        if(parent == null){
            throw new RemoteServiceException("请选择父部门");
        }
        if(StringUtils.equals(adminWebsit.getWebsitId(),adminWebsit.getParentId())){
            throw new RemoteServiceException("父级部门不可以是自己");
        }
        List<AdminWebsit> list = new ArrayList<>();
        list.addAll(commonLogic.queryAllChild(list,adminWebsit.getWebsitId()));
        List<String> childWebsitIds = list.stream().map(AdminWebsit::getWebsitId).collect(Collectors.toList());
        if(childWebsitIds.contains(adminWebsit.getParentId())){
            throw new RemoteServiceException("父级部门不可以是自己或者自己的下级部门");
        }
        adminWebsit.setType(parent.getType());
        adminWebsitService.saveOrUpdate(adminWebsit);
        //workWechatLogic.updateOrganization(adminWebsit.getWebsitId(),adminWebsit.getName(),adminWebsit.getParentId(),adminUser.getAdminCompanyWechat());
    }

    /**
     * 删除部门
     */
    @Transactional
    public void delete(String websitId,HttpServletRequest request) throws WxErrorException {
        AdminWebsit old = adminWebsitService.getById(websitId);
        if(Constant.SYS_WEBSIT_NAME.contains(old.getName())){
            throw new RemoteServiceException("不可操作当前部门信息");
        }
        Long count = adminWebsitService.lambdaQuery().eq(AdminWebsit::getParentId, websitId).count();
        if(count > 0){
            throw new RemoteServiceException("当前部门存在子级，删除失败");
        }
        Long count1 = adminUserService.lambdaQuery().eq(AdminUser::getAdminWebsitId, websitId).count();
        if(count1 > 0){
            throw new RemoteServiceException("当前部门存在账号，删除失败");
        }
        AdminUserCom adminUser = commonLogic.getAdminUser(request);
        //递归删除
        List<AdminWebsit> list = new ArrayList<>();
        list.addAll(commonLogic.queryAllChild(list,websitId));
        List<String> ids = list.stream().map(AdminWebsit::getWebsitId).collect(Collectors.toList());
        ids.add(websitId);
        if(ids.size() > 0) {
//            for(String idd : ids){
//                workWechatLogic.deleteOrganization(idd,adminUser.getAdminCompanyWechat());
//            }
            adminWebsitService.lambdaUpdate().in(AdminWebsit::getWebsitId, ids).remove();
        }
    }

    /**
     * 详情
     * @param id
     */
    public AdminWebsit detail(String id){
        AdminWebsit adminWebsit = adminWebsitService.getById(id);
        return adminWebsit;
    }

    /**
     * 递归组装数据
     * @param list
     * @param parentId
     * @return
     */
    private List<AdminWebsitTree> treeModule(List<AdminWebsitTree> list, String parentId){
        //最父级资源树
        if(parentId == null || parentId.equals("0")) {
            return null;
        }
        List<AdminWebsitTree> collect = list.stream()
                .filter(e -> e.getParentId().equals(parentId)).collect(Collectors.toList());
        if(collect.size() == 0){
            return null;
        }
        for(AdminWebsitTree bean : collect){
            bean.setChildren(this.treeModule(list, bean.getWebsitId()));
        }
        return collect;
    }



    /**
     * 保存部门的服务区域
     * @param adminWebsitRegions
     */
    @Transactional
    public void saveRegion(List<AdminWebsitRegion> adminWebsitRegions){
        List<String> adminWebsitIds = adminWebsitRegions.stream().map(AdminWebsitRegion::getAdminWebsitId).distinct().collect(Collectors.toList());
        if(adminWebsitIds.size() != 1){
            throw new RemoteServiceException("参数异常，请检查所有部门id是否一致");
        }
//        if(adminWebsitRegions.size() > 1){
//            throw new RemoteServiceException("暂只支持1个个保存");
//        }
        for(AdminWebsitRegion adminWebsitRegion : adminWebsitRegions) {
            Long count = adminWebsitRegionService.lambdaQuery()
                    .eq(AdminWebsitRegion::getAdminWebsitId, adminWebsitRegion.getAdminWebsitId())
                    .eq(AdminWebsitRegion::getServiceCategoryId, adminWebsitRegion.getServiceCategoryId())
                    .eq(AdminWebsitRegion::getProvinceId, adminWebsitRegion.getProvinceId())
                    .eq(AdminWebsitRegion::getCityId, adminWebsitRegion.getCityId())
                    .eq(AdminWebsitRegion::getAreaId, adminWebsitRegion.getAreaId())
                    .eq(AdminWebsitRegion::getStreetId, adminWebsitRegion.getStreetId())
                    .count();
            String pcas = adminWebsitRegion.getProvince() + adminWebsitRegion.getCity() + adminWebsitRegion.getArea() + adminWebsitRegion.getStreet();
            if (count > 0) {
                throw new RemoteServiceException(pcas + "已存在，请勿重复添加");
            }
            adminWebsitRegion.setPcas(pcas);
            adminWebsitRegionService.save(adminWebsitRegion);
        }
        //刷新服务商的服务类目冗余字段
        reloadServiceCategoryName(adminWebsitRegions.get(0).getAdminWebsitId());
    }

    /**
     * 部门的服务区域列表
     * @param adminWebsitId
     * @return
     */
    public List<AdminWebsitRegion> listRegion(String adminWebsitId,String keyword,String serviceCategoryId){
        List<AdminWebsitRegion> list = adminWebsitRegionService.lambdaQuery()
                .like(StringUtils.isNotBlank(keyword),AdminWebsitRegion::getPcas,keyword)
                .eq(StringUtils.isNotBlank(serviceCategoryId),AdminWebsitRegion::getServiceCategoryId,serviceCategoryId)
                .eq(AdminWebsitRegion::getAdminWebsitId, adminWebsitId).list();
        return list;
    }

    /**
     * 删除服务区域
     */
    public void deleteRegion(String id){
        AdminWebsitRegion adminWebsitRegion = adminWebsitRegionService.getById(id);
        adminWebsitRegionService.removeById(id);
        //刷新服务商的服务类目冗余字段
        reloadServiceCategoryName(adminWebsitRegion.getAdminWebsitId());
    }


    /**
     * 刷新服务商的服务类目id
     * @param adminWebsitId
     * @return
     */
    public void reloadServiceCategoryName(String adminWebsitId){
        String serviceCategoryName = adminWebsitRegionService.lambdaQuery().eq(AdminWebsitRegion::getAdminWebsitId, adminWebsitId)
                .list().stream()
                .map(AdminWebsitRegion::getServiceCategoryName)
                .distinct()
                .collect(Collectors.joining());

        serviceProviderService.lambdaUpdate()
                .set(ServiceProvider::getServiceCategoryName,serviceCategoryName)
                .eq(ServiceProvider::getId,adminWebsitId).update();
    }








}
